@@ -171,6 +171,11 @@ enum ErrorDetail {
     #[cfg(feature = "onion-service-client")]
     HsClientConnectorSetup(#[from] tor_hsclient::StartupError),
 
+    /// Error setting up onion service.
+    #[cfg(feature= "onion-service-service")]
+    #[error("Error setting up onion service")]
+    OnionServiceSetup(#[source] tor_hsservice::StartupError),
+
     /// Failed to obtain exit circuit
     #[error("Failed to obtain exit circuit for ports {exit_ports}")]
     ObtainExitCircuit {
@@ -317,6 +322,8 @@ enum ErrorDetail {
     BadClientSpecifier(#[from] tor_keymgr::ArtiPathSyntaxError),
 
     /// We tried to parse an onion address, but we found that it was invalid.
+    ///
+    /// This error occurs if we are asked to connect to an invalid .onion address.
     #[cfg(feature = "onion-service-client")]
     #[error("Invalid onion address")]
     BadOnionAddress(#[from] tor_hscrypto::pk::HsIdParseError),
@@ -327,6 +334,10 @@ enum ErrorDetail {
     #[error("Unable to launch onion service")]
     LaunchOnionService(#[source] tor_hsservice::StartupError),
 
+    /// We found that at least one required protocol was missing.
+    #[error("Arti is missing a required protocol feature")]
+    MissingProtocol(#[source] tor_netdoc::doc::netstatus::ProtocolSupportError),
+
     /// A programming problem, either in our code or the code calling it.
     #[error("Programming problem")]
     Bug(#[from] tor_error::Bug),
@@ -410,6 +421,8 @@ impl tor_error::HasKind for ErrorDetail {
             E::StateMgrSetup(e) => e.kind(),
             #[cfg(feature = "onion-service-client")]
             E::HsClientConnectorSetup(e) => e.kind(),
+            #[cfg(feature = "onion-service-service")]
+            E::OnionServiceSetup(e) => e.kind(),
             E::DirMgrBootstrap(e) => e.kind(),
             #[cfg(feature = "pt-client")]
             E::PluggableTransport(e) => e.kind(),
@@ -423,7 +436,7 @@ impl tor_error::HasKind for ErrorDetail {
             #[cfg(feature = "onion-service-client")]
             E::OnionAddressDisabled => EK::ForbiddenStreamTarget,
             #[cfg(feature = "onion-service-client")]
-            E::BadOnionAddress(e) => e.kind(),
+            E::BadOnionAddress(_) => EK::InvalidStreamTarget,
             #[cfg(feature = "onion-service-service")]
             E::LaunchOnionService(e) => e.kind(),
             // TODO Should delegate to TorAddrError EK
@@ -435,6 +448,7 @@ impl tor_error::HasKind for ErrorDetail {
             E::KeystoreRequired { .. } => EK::InvalidConfig,
             E::BadClientSpecifier(_) => EK::InvalidConfig,
             E::FsMistrust(_) => EK::FsPermissions,
+            E::MissingProtocol(_) => EK::SoftwareDeprecated,
             E::Bug(e) => e.kind(),
         }
     }
@@ -497,6 +511,12 @@ enum ErrorHintInner<'a> {
         /// The access bits that, according to fs-mistrust, should not be set.
         badbits: u32,
     },
+
+    /// At least one required protocol was missing.
+    MissingProtocols {
+        /// The list of missing required protocols
+        required: &'a tor_protover::Protocols,
+    },
 }
 
 // TODO: Perhaps we want to lower this logic to fs_mistrust crate, and have a
@@ -536,6 +556,15 @@ impl<'a> Display for ErrorHint<'a> {
                 writeln!(f, "You can suppress this message by setting storage.permissions.dangerously_trust_everyone=true,\n\
                     or setting ARTI_FS_DISABLE_PERMISSION_CHECKS=yes in your environment.")?;
             }
+            ErrorHintInner::MissingProtocols { required } => {
+                writeln!(f, "The consensus directory says that we need to support certain protocols which we do not implement.")?;
+                writeln!(f, "The missing protocols are: {}", required)?;
+                writeln!(
+                    f,
+"The best solution is to upgrade to a more recent version of Arti.  If this is not possible,
+you can list the missing protocols in the configuration option 'use_obsolete_software.ignore_missing_required_protocols'"
+                )?;
+            }
         }
         Ok(())
     }
