@@ -52,6 +52,11 @@ pub mod llconn;
 mod msgs;
 #[macro_use]
 mod util;
+#[cfg(test)]
+mod testing;
 
-pub use conn::{BuilderError, ConnectError, ProtoError, RpcConn, RpcConnBuilder, StreamError};
+pub use conn::{
+    BuilderError, ConnPtDescription, ConnectError, ConnectFailure, ProtoError, RpcConn,
+    RpcConnBuilder, StreamError,
+};
 pub use msgs::{request::InvalidRequestError, response::RpcError, AnyRequestId, ObjectId};
