@@ -34,11 +34,13 @@
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
+#![deny(clippy::mod_module_files)]
 #![allow(clippy::let_unit_value)] // This can reasonably be done for explicitness
 #![allow(clippy::uninlined_format_args)]
 #![allow(clippy::significant_drop_in_scrutinee)] // arti/-/merge_requests/588/#note_2812945
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
+#![allow(clippy::needless_lifetimes)] // See arti#1765
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 use derive_more::Display;
@@ -268,6 +270,15 @@ pub enum ErrorKind {
     #[display("Tor client is shutting down.")]
     ArtiShuttingDown,
 
+    /// This Tor client software is missing some feature that is recommended
+    /// (or required) for operation on the network.
+    ///
+    /// This occurs when the directory authorities tell us that we ought to have
+    /// a particular protocol feature that we do not support.
+    /// The correct solution is likely to upgrade to a more recent version of Arti.
+    #[display("Software version is deprecated")]
+    SoftwareDeprecated,
+
     /// An operation failed because we waited too long for an exit to do
     /// something.
     ///
