@@ -47,13 +47,12 @@
 #![cfg_attr(not(all(feature = "full")), allow(unused))]
 
 pub use crate::err::Error;
-use once_cell::sync::OnceCell;
 use rangemap::RangeInclusiveMap;
 use std::fmt::{Debug, Display, Formatter};
 use std::net::{IpAddr, Ipv6Addr};
 use std::num::{NonZeroU32, NonZeroU8, TryFromIntError};
 use std::str::FromStr;
-use std::sync::Arc;
+use std::sync::{Arc, OnceLock};
 
 mod err;
 
@@ -70,7 +69,7 @@ static EMBEDDED_DB_V6: &str = include_str!("../data/geoip6");
 
 /// A parsed copy of the embedded database.
 #[cfg(feature = "embedded-db")]
-static EMBEDDED_DB_PARSED: OnceCell<Arc<GeoipDb>> = OnceCell::new();
+static EMBEDDED_DB_PARSED: OnceLock<Arc<GeoipDb>> = OnceLock::new();
 
 /// A two-letter country code.
 ///
