@@ -129,8 +129,8 @@ pub(crate) use {
     crate::keys::expire_publisher_keys,
     crate::keys::{IptKeyRole, IptKeySpecifier, IptKeySpecifierPattern},
     crate::publish::Publisher,
+    crate::replay::IptReplayLog,
     crate::replay::ReplayError,
-    crate::replay::ReplayLog,
     crate::status::PublisherStatusSender,
     crate::status::State,
     crate::status::{IptMgrStatusSender, State as IptMgrState},
