@@ -34,11 +34,13 @@
 #![deny(clippy::unnecessary_wraps)]
 #![warn(clippy::unseparated_literal_suffix)]
 #![deny(clippy::unwrap_used)]
+#![deny(clippy::mod_module_files)]
 #![allow(clippy::let_unit_value)] // This can reasonably be done for explicitness
 #![allow(clippy::uninlined_format_args)]
 #![allow(clippy::significant_drop_in_scrutinee)] // arti/-/merge_requests/588/#note_2812945
 #![allow(clippy::result_large_err)] // temporary workaround for arti#587
 #![allow(clippy::needless_raw_string_hashes)] // complained-about code is fine, often best
+#![allow(clippy::needless_lifetimes)] // See arti#1765
 //! <!-- @@ end lint list maintained by maint/add_warning @@ -->
 
 // TODO #1645 (either remove this, or decide to have it everywhere)
@@ -358,8 +360,16 @@ impl OnionService {
     ///
     /// Returns `None` if the HsId of the service could not be found in any of the configured
     /// keystores.
+    pub fn onion_address(&self) -> Option<HsId> {
+        onion_address(&self.keymgr, &self.config.nickname)
+    }
+
+    /// Return the onion address of this service.
+    ///
+    /// See [`onion_address`](Self::onion_address)
+    #[deprecated = "Use the new onion_address method instead"]
     pub fn onion_name(&self) -> Option<HsId> {
-        onion_name(&self.keymgr, &self.config.nickname)
+        self.onion_address()
     }
 
     /// Generate an identity key (KP_hs_id) for this service.
@@ -495,8 +505,16 @@ impl RunningOnionService {
     ///
     /// Returns `None` if the HsId of the service could not be found in any of the configured
     /// keystores.
+    pub fn onion_address(&self) -> Option<HsId> {
+        onion_address(&self.keymgr, &self.nickname)
+    }
+
+    /// Return the onion address of this service.
+    ///
+    /// See [`onion_address`](Self::onion_address)
+    #[deprecated = "Use the new onion_address method instead"]
     pub fn onion_name(&self) -> Option<HsId> {
-        onion_name(&self.keymgr, &self.nickname)
+        self.onion_address()
     }
 }
 
@@ -522,7 +540,7 @@ fn maybe_generate_hsid(
             cause,
         })?;
 
-    let mut rng = rand::thread_rng();
+    let mut rng = tor_llcrypto::rng::CautiousRng;
     let (hsid, generated) = match kp {
         Some(kp) => (kp.id(), false),
         None => {
@@ -566,10 +584,10 @@ fn maybe_generate_hsid(
 /// Returns `None` if the HsId of the service could not be found in any of the configured
 /// keystores.
 //
-// TODO: instead of duplicating RunningOnionService::onion_name, maybe we should make this a
+// TODO: instead of duplicating RunningOnionService::onion_address, maybe we should make this a
 // method on an ArtiHss type, and make both OnionService and RunningOnionService deref to
 // ArtiHss.
-fn onion_name(keymgr: &KeyMgr, nickname: &HsNickname) -> Option<HsId> {
+fn onion_address(keymgr: &KeyMgr, nickname: &HsNickname) -> Option<HsId> {
     let hsid_spec = HsIdPublicKeySpecifier::new(nickname.clone());
 
     keymgr
@@ -578,6 +596,23 @@ fn onion_name(keymgr: &KeyMgr, nickname: &HsNickname) -> Option<HsId> {
         .map(|hsid| hsid.id())
 }
 
+/// Return a list of the protocols[supported](tor_protover::doc_supported)
+/// by this crate, running as a hidden service.
+pub fn supported_hsservice_protocols() -> tor_protover::Protocols {
+    use tor_protover::named::*;
+    // WARNING: REMOVING ELEMENTS FROM THIS LIST CAN BE DANGEROUS!
+    // SEE [`tor_protover::doc_changing`]
+    [
+        //
+        HSINTRO_V3,
+        HSINTRO_RATELIM,
+        HSREND_V3,
+        HSDIR_V3,
+    ]
+    .into_iter()
+    .collect()
+}
+
 #[cfg(test)]
 pub(crate) mod test {
     // @@ begin test lint list maintained by maint/add_warning @@
@@ -613,6 +648,13 @@ pub(crate) mod test {
     /// The nickname of the test service.
     const TEST_SVC_NICKNAME: &str = "test-svc";
 
+    #[test]
+    fn protocols() {
+        let pr = supported_hsservice_protocols();
+        let expected = "HSIntro=4-5 HSRend=2 HSDir=2".parse().unwrap();
+        assert_eq!(pr, expected);
+    }
+
     /// Make a fresh `KeyMgr` (containing no keys) using files in `temp_dir`
     pub(crate) fn create_keymgr(temp_dir: &TestTempDir) -> TestTempDirGuard<Arc<KeyMgr>> {
         temp_dir.subdir_used_by("keystore", |keystore_dir| {
@@ -782,7 +824,7 @@ pub(crate) mod test {
     }
 
     #[test]
-    fn onion_name() {
+    fn onion_address() {
         let temp_dir = test_temp_dir!();
         let nickname = HsNickname::try_from(TEST_SVC_NICKNAME.to_string()).unwrap();
         let hsid_spec = HsIdKeypairSpecifier::new(nickname.clone());
@@ -814,7 +856,7 @@ pub(crate) mod test {
             .unwrap();
 
         let hsid = HsId::from(hsid_public);
-        assert_eq!(service.onion_name().unwrap(), hsid);
+        assert_eq!(service.onion_address().unwrap(), hsid);
 
         drop(temp_dir); // prove that this is still live
     }
