@@ -251,12 +251,7 @@ impl Keystore for CTorClientKeystore {
             .map(|k: curve25519::StaticKeypair| Box::new(k) as ErasedKey))
     }
 
-    fn insert(
-        &self,
-        _key: &dyn EncodableItem,
-        _key_spec: &dyn KeySpecifier,
-        _item_type: &KeystoreItemType,
-    ) -> Result<()> {
+    fn insert(&self, _key: &dyn EncodableItem, _key_spec: &dyn KeySpecifier) -> Result<()> {
         Err(CTorKeystoreError::NotSupported { action: "insert" }.into())
     }
 
@@ -404,11 +399,7 @@ mod tests {
         assert_eq!(err.to_string(), "Operation not supported: remove");
 
         let err = keystore
-            .insert(
-                &DummyKey,
-                &TestCTorSpecifier(path),
-                &KeyType::X25519StaticKeypair.into(),
-            )
+            .insert(&DummyKey, &TestCTorSpecifier(path))
             .unwrap_err();
 
         assert_eq!(err.to_string(), "Operation not supported: insert");
