@@ -71,6 +71,15 @@ impl Keystore for ArtiEphemeralKeystore {
             .map_err(ArtiEphemeralKeystoreError::ArtiPathUnavailableError)?;
         let key_dictionary = self.key_dictionary.lock().expect("lock poisoned");
         match key_dictionary.get(&(arti_path.clone(), item_type.clone())) {
+            Some(key) if key.item_type()? != *item_type => {
+                // This should only happen if some external factor alters the
+                // process memory or if there's a bug in our implementation of
+                // Keystore::insert().
+                Err(internal!(
+                    "the specified KeystoreItemType does not match key type of the fetched key?!"
+                )
+                .into())
+            }
             Some(key) => {
                 let key: KeystoreItem = key.clone();
                 let key: ErasedKey = key.into_erased()?;
@@ -80,38 +89,16 @@ impl Keystore for ArtiEphemeralKeystore {
         }
     }
 
-    fn insert(
-        &self,
-        key: &dyn EncodableItem,
-        key_spec: &dyn KeySpecifier,
-        item_type: &KeystoreItemType,
-    ) -> Result<(), Error> {
+    fn insert(&self, key: &dyn EncodableItem, key_spec: &dyn KeySpecifier) -> Result<(), Error> {
         let arti_path = key_spec
             .arti_path()
             .map_err(ArtiEphemeralKeystoreError::ArtiPathUnavailableError)?;
         let key_data = key.as_keystore_item()?;
-
-        // TODO: add item_type validation to Keystore::get and Keystore::remove.
-        // The presence of a key with a mismatched item_type can be either due to keystore
-        // corruption, or API misuse. We will need a new error type and corresponding ErrorKind for
-        // that).
-        //
-        // TODO: add item_type validation to ArtiNativeKeystore
-        if key_data.item_type()? != *item_type {
-            // This can never happen unless:
-            //   * Keystore::insert is called directly with an incorrect KeystoreItemType for `key`, or
-            //   * Keystore::insert is called via KeyMgr, but the EncodableItem implementation of
-            //   the key is broken. EncodableItem can't be implemented by external types,
-            //   so a broken implementation means we have an internal bug.
-            return Err(internal!(
-                "the specified KeystoreItemType does not match key type of the inserted key?!"
-            )
-            .into());
-        }
+        let item_type = key_data.item_type()?;
 
         // save to dictionary
         let mut key_dictionary = self.key_dictionary.lock().expect("lock poisoned");
-        let _ = key_dictionary.insert((arti_path, item_type.clone()), key_data);
+        let _ = key_dictionary.insert((arti_path, item_type), key_data);
         Ok(())
     }
 
@@ -124,9 +111,19 @@ impl Keystore for ArtiEphemeralKeystore {
             .arti_path()
             .map_err(ArtiEphemeralKeystoreError::ArtiPathUnavailableError)?;
         let mut key_dictionary = self.key_dictionary.lock().expect("lock poisoned");
-        Ok(key_dictionary
-            .remove(&(arti_path, item_type.clone()))
-            .map(|_| ()))
+        match key_dictionary.remove(&(arti_path, item_type.clone())) {
+            Some(key) if key.item_type()? != *item_type => {
+                // This should only happen if some external factor alters the
+                // process memory or if there's a bug in our implementation of
+                // Keystore::insert().
+                Err(internal!(
+                    "the specified KeystoreItemType does not match key type of the removed key?!"
+                )
+                .into())
+            }
+            Some(_) => Ok(Some(())),
+            None => Ok(None),
+        }
     }
 
     fn list(&self) -> Result<Vec<(KeyPath, KeystoreItemType)>, Error> {
@@ -154,9 +151,11 @@ mod tests {
     #![allow(clippy::needless_pass_by_value)]
     //! <!-- @@ end test lint list maintained by maint/add_warning @@ -->
 
-    use tor_basic_utils::test_rng::testing_rng;
-    use tor_key_forge::KeyType;
-    use tor_llcrypto::pk::ed25519;
+    use tor_basic_utils::test_rng::{testing_rng, TestingRng};
+    use tor_error::{ErrorKind, HasKind};
+    use tor_key_forge::{KeyType, Keygen};
+    use tor_llcrypto::pk::{curve25519, ed25519};
+    use tor_llcrypto::rng::FakeEntropicRng;
 
     use super::*;
 
@@ -174,6 +173,12 @@ mod tests {
         KeyType::Ed25519Keypair.into()
     }
 
+    fn key_bad() -> Box<dyn EncodableItem> {
+        let mut rng = FakeEntropicRng::<TestingRng>(testing_rng());
+        let keypair = curve25519::StaticKeypair::generate(&mut rng).unwrap();
+        Box::new(keypair)
+    }
+
     fn key_type_bad() -> KeystoreItemType {
         KeyType::X25519StaticKeypair.into()
     }
@@ -202,7 +207,7 @@ mod tests {
 
         // insert key and verify in store
         assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type())
+            .insert(key().as_ref(), key_spec().as_ref())
             .is_ok());
         assert!(key_store
             .contains(key_spec().as_ref(), &key_type())
@@ -221,7 +226,7 @@ mod tests {
 
         // insert and verify get is a result
         assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type())
+            .insert(key().as_ref(), key_spec().as_ref())
             .is_ok());
 
         let key = key_store
@@ -231,17 +236,30 @@ mod tests {
 
         // Ensure the returned key is of the right type
         assert!(key.downcast::<ed25519::Keypair>().is_ok());
+
+        // verify receiving a key of a different type results in the appropriate error
+        key_store.remove(key_spec().as_ref(), &key_type()).unwrap();
+        {
+            let mut key_dictionary = key_store.key_dictionary.lock().unwrap();
+            let _ = key_dictionary.insert(
+                (key_spec().arti_path().unwrap(), key_type()),
+                key_bad().as_keystore_item().unwrap(),
+            );
+        }
+        assert!(matches!(
+            key_store
+                .get(key_spec().as_ref(), &key_type())
+                .err()
+                .unwrap()
+                .kind(),
+            ErrorKind::Internal
+        ));
     }
 
     #[test]
     fn insert() {
         let key_store = ArtiEphemeralKeystore::new("test-ephemeral".to_string());
 
-        // verify inserting a key with the wrong key type fails
-        assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type_bad())
-            .is_err());
-        // further ensure there are no side effects
         assert!(!key_store
             .contains(key_spec().as_ref(), &key_type_bad())
             .unwrap());
@@ -251,9 +269,9 @@ mod tests {
             .is_none());
         assert!(key_store.list().unwrap().is_empty());
 
-        // verify inserting a good key succeeds
+        // verify inserting a key succeeds
         assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type())
+            .insert(key().as_ref(), key_spec().as_ref())
             .is_ok());
 
         // further ensure correct side effects
@@ -279,12 +297,29 @@ mod tests {
 
         // verify inserting and removing results in Some(())
         assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type())
+            .insert(key().as_ref(), key_spec().as_ref())
             .is_ok());
         assert!(key_store
             .remove(key_spec().as_ref(), &key_type())
             .unwrap()
             .is_some());
+
+        // verify mismatched key type on removal results in the appropriate error
+        {
+            let mut key_dictionary = key_store.key_dictionary.lock().unwrap();
+            let _ = key_dictionary.insert(
+                (key_spec().arti_path().unwrap(), key_type()),
+                key_bad().as_keystore_item().unwrap(),
+            );
+        }
+        assert!(matches!(
+            key_store
+                .remove(key_spec().as_ref(), &key_type())
+                .err()
+                .unwrap()
+                .kind(),
+            ErrorKind::Internal
+        ));
     }
 
     #[test]
@@ -296,7 +331,7 @@ mod tests {
 
         // verify size 1 after inserting a key
         assert!(key_store
-            .insert(key().as_ref(), key_spec().as_ref(), &key_type())
+            .insert(key().as_ref(), key_spec().as_ref())
             .is_ok());
         assert_eq!(key_store.list().unwrap().len(), 1);
     }
