@@ -13,7 +13,7 @@ use std::iter;
 use std::result::Result as StdResult;
 use tor_error::{bad_api_usage, internal, into_bad_api_usage};
 use tor_key_forge::{
-    EncodableItem, Keygen, KeygenRng, KeystoreItemType, ToEncodableCert, ToEncodableKey,
+    ItemType, Keygen, KeygenRng, KeystoreItemType, ToEncodableCert, ToEncodableKey,
 };
 
 /// A key manager that acts as a frontend to a primary [`Keystore`](crate::Keystore) and
@@ -223,11 +223,10 @@ impl KeyMgr {
         K::Key: Keygen,
     {
         let store = self.select_keystore(&selector)?;
-        let key_type = K::Key::item_type();
 
-        if overwrite || !store.contains(key_spec, &key_type)? {
+        if overwrite || !store.contains(key_spec, &K::Key::item_type())? {
             let key = K::Key::generate(rng)?;
-            store.insert(&key, key_spec, &key_type)?;
+            store.insert(&key, key_spec)?;
 
             Ok(K::from_encodable_key(key))
         } else {
@@ -262,7 +261,7 @@ impl KeyMgr {
         if old_key.is_some() && !overwrite {
             Err(crate::Error::KeyAlreadyExists)
         } else {
-            let () = store.insert(&key, key_spec, &key_type)?;
+            let () = store.insert(&key, key_spec)?;
             Ok(old_key)
         }
     }
@@ -353,7 +352,7 @@ impl KeyMgr {
     /// Returns the `<K as ToEncodableKey>::Key` representation of the key.
     ///
     /// See [`KeyMgr::get`] for more details.
-    fn get_from_store_raw<'a, K: EncodableItem>(
+    fn get_from_store_raw<'a, K: ItemType>(
         &self,
         key_spec: &dyn KeySpecifier,
         key_type: &KeystoreItemType,
@@ -449,9 +448,9 @@ impl KeyMgr {
             ArtiPath::from_path_and_denotators(subject_key_arti_path, &spec.cert_denotators())
                 .map_err(into_bad_api_usage!("invalid certificate specifier"))?;
 
-        let Some(cert) = self.get_from_store_raw::<C::Cert>(
+        let Some(cert) = self.get_from_store_raw::<C::ParsedCert>(
             &cert_spec,
-            &C::Cert::item_type(),
+            &<C::ParsedCert as ItemType>::item_type(),
             self.all_stores(),
         )?
         else {
@@ -460,8 +459,7 @@ impl KeyMgr {
 
         // Finally, get the signing key and validate the cert
         let signed_with = self.get_cert_signing_key::<K, C>(spec)?;
-        let cert = C::from_encodable_cert(cert);
-        cert.validate(&key, &signed_with)?;
+        let cert = C::validate(cert, &key, &signed_with)?;
 
         Ok(Some((key, cert)))
     }
@@ -523,9 +521,9 @@ impl KeyMgr {
             ArtiPath::from_path_and_denotators(subject_key_arti_path, &spec.cert_denotators())
                 .map_err(into_bad_api_usage!("invalid certificate specifier"))?;
 
-        let maybe_cert = self.get_from_store_raw::<C::Cert>(
+        let maybe_cert = self.get_from_store_raw::<C::ParsedCert>(
             &cert_specifier,
-            &C::Cert::item_type(),
+            &C::ParsedCert::item_type(),
             self.all_stores(),
         )?;
 
@@ -546,7 +544,7 @@ impl KeyMgr {
 
         let signed_with = self.get_cert_signing_key::<K, C>(spec)?;
         let cert = match maybe_cert {
-            Some(cert) => C::from_encodable_cert(cert),
+            Some(cert) => C::validate(cert, &subject_key, &signed_with)?,
             None => {
                 let cert = make_certificate(&subject_key, &signed_with);
 
@@ -556,7 +554,6 @@ impl KeyMgr {
             }
         };
 
-        cert.validate(&subject_key, &signed_with)?;
         Ok((subject_key, cert))
     }
 
@@ -637,9 +634,8 @@ impl KeyMgr {
     {
         let cert = cert.to_encodable_cert();
         let store = self.select_keystore(&selector)?;
-        let cert_type = C::Cert::item_type();
 
-        let () = store.insert(&cert, cert_spec, &cert_type)?;
+        let () = store.insert(&cert, cert_spec)?;
         Ok(())
     }
 }
@@ -665,19 +661,36 @@ mod tests {
     use std::result::Result as StdResult;
     use std::str::FromStr;
     use std::sync::RwLock;
+    use std::time::{Duration, SystemTime};
     use tor_basic_utils::test_rng::testing_rng;
+    use tor_cert::CertifiedKey;
+    use tor_cert::Ed25519Cert;
     use tor_key_forge::{
         CertData, EncodableItem, ErasedKey, InvalidCertError, KeyType, KeystoreItem,
     };
-    use tor_llcrypto::pk::ed25519;
+    use tor_llcrypto::pk::ed25519::{self, Ed25519PublicKey as _};
+    use tor_llcrypto::rng::FakeEntropicRng;
+
+    /// Metadata structure for tracking item operations in tests.
+    #[derive(Clone, Debug, PartialEq)]
+    struct ItemMetadata {
+        /// The identifier for the item (e.g., "coot", "moorhen").
+        item_id: String,
+        /// The keystore from which the item was retrieved.
+        ///
+        /// Set by `Keystore::get`.
+        retrieved_from: Option<KeystoreId>,
+        /// Whether the item was generated via `Keygen::generate`.
+        is_generated: bool,
+    }
 
     /// The type of "key" stored in the test key stores.
     #[derive(Clone, Debug)]
     struct TestItem {
         /// The underlying key.
         item: KeystoreItem,
-        /// Some metadata about the key
-        meta: String,
+        /// Metadata about the key.
+        meta: ItemMetadata,
     }
 
     /// A "certificate" used for testing purposes.
@@ -699,13 +712,17 @@ mod tests {
 
     impl TestItem {
         /// Create a new test key with the specified metadata.
-        fn new(meta: &str) -> Self {
+        fn new(item_id: &str) -> Self {
             let mut rng = testing_rng();
             TestItem {
                 item: ed25519::Keypair::generate(&mut rng)
                     .as_keystore_item()
                     .unwrap(),
-                meta: meta.into(),
+                meta: ItemMetadata {
+                    item_id: item_id.to_string(),
+                    retrieved_from: None,
+                    is_generated: false,
+                },
             }
         }
     }
@@ -717,12 +734,16 @@ mod tests {
         {
             Ok(TestItem {
                 item: ed25519::Keypair::generate(&mut rng).as_keystore_item()?,
-                meta: "generated_test_key".into(),
+                meta: ItemMetadata {
+                    item_id: "generated_test_key".to_string(),
+                    retrieved_from: None,
+                    is_generated: true,
+                },
             })
         }
     }
 
-    impl EncodableItem for TestItem {
+    impl ItemType for TestItem {
         fn item_type() -> KeystoreItemType
         where
             Self: Sized,
@@ -730,7 +751,9 @@ mod tests {
             // Dummy value
             KeyType::Ed25519Keypair.into()
         }
+    }
 
+    impl EncodableItem for TestItem {
         fn as_keystore_item(&self) -> tor_key_forge::Result<KeystoreItem> {
             Ok(self.item.clone())
         }
@@ -749,14 +772,16 @@ mod tests {
         }
     }
 
-    impl EncodableItem for TestPublicKey {
+    impl ItemType for TestPublicKey {
         fn item_type() -> KeystoreItemType
         where
             Self: Sized,
         {
             KeyType::Ed25519PublicKey.into()
         }
+    }
 
+    impl EncodableItem for TestPublicKey {
         fn as_keystore_item(&self) -> tor_key_forge::Result<KeystoreItem> {
             Ok(self.key.clone())
         }
@@ -776,30 +801,23 @@ mod tests {
     }
 
     impl ToEncodableCert<TestItem> for AlwaysValidCert {
-        type Cert = TestItem;
+        type ParsedCert = TestItem;
+        type EncodableCert = TestItem;
         type SigningKey = TestItem;
 
         fn validate(
-            &self,
+            cert: Self::ParsedCert,
             _subject: &TestItem,
             _signed_with: &Self::SigningKey,
-        ) -> StdResult<(), InvalidCertError> {
+        ) -> StdResult<Self, InvalidCertError> {
             // AlwaysValidCert is always valid
-            Ok(())
+            Ok(Self(cert))
         }
 
         /// Convert this cert to a type that implements [`EncodableKey`].
-        fn to_encodable_cert(self) -> Self::Cert {
+        fn to_encodable_cert(self) -> Self::EncodableCert {
             self.0
         }
-
-        /// Convert an [`EncodableKey`] to another cert type.
-        fn from_encodable_cert(cert: Self::Cert) -> Self
-        where
-            Self: Sized,
-        {
-            Self(cert)
-        }
     }
 
     macro_rules! impl_keystore {
@@ -852,26 +870,30 @@ mod tests {
                         .read()
                         .unwrap()
                         .get(&(key_spec.arti_path().unwrap(), item_type.clone()))
-                        .map(|k| Box::new(k.clone()) as Box<dyn EncodableItem>))
+                        .map(|k| {
+                            let mut k = k.clone();
+                            k.meta.retrieved_from = Some(self.id().clone());
+                            Box::new(k) as Box<dyn ItemType>
+                        }))
                 }
 
                 fn insert(
                     &self,
                     key: &dyn EncodableItem,
                     key_spec: &dyn KeySpecifier,
-                    item_type: &KeystoreItemType,
                 ) -> Result<()> {
                     let key = key.downcast_ref::<TestItem>().unwrap();
-                    let value = &key.meta;
-                    let key = TestItem {
-                        item: key.as_keystore_item()?,
-                        meta: format!("{}_{value}", self.id()),
-                    };
+
+                    let item = key.as_keystore_item()?;
+                    let meta = key.meta.clone();
+
+                    let item_type = item.item_type()?;
+                    let key = TestItem { item, meta };
 
                     self.inner
                         .write()
                         .unwrap()
-                        .insert((key_spec.arti_path().unwrap(), item_type.clone()), key);
+                        .insert((key_spec.arti_path().unwrap(), item_type), key);
 
                     Ok(())
                 }
@@ -945,6 +967,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::cognitive_complexity)]
     fn insert_and_get() {
         let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
@@ -965,12 +988,13 @@ mod tests {
             .unwrap();
 
         assert!(old_key.is_none());
+        let key = mgr.get::<TestItem>(&TestKeySpecifier1).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "coot");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier1)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore2_coot".to_string()),
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
 
         // Insert a different key using the _same_ key specifier.
         let old_key = mgr
@@ -981,16 +1005,21 @@ mod tests {
                 true,
             )
             .unwrap()
-            .unwrap()
-            .meta;
-        assert_eq!(old_key, "keystore2_coot");
-        // Check that the original value was overwritten:
+            .unwrap();
+        assert_eq!(old_key.meta.item_id, "coot");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier1)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore2_gull".to_string()),
+            old_key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
         );
+        assert_eq!(old_key.meta.is_generated, false);
+        // Check that the original value was overwritten:
+        let key = mgr.get::<TestItem>(&TestKeySpecifier1).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "gull");
+        assert_eq!(
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
+        );
+        assert_eq!(key.meta.is_generated, false);
 
         // Insert a different key using the _same_ key specifier (overwrite = false)
         let err = mgr
@@ -1024,12 +1053,13 @@ mod tests {
             )
             .unwrap();
         assert!(old_key.is_none());
+        let key = mgr.get::<TestItem>(&TestKeySpecifier3).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "moorhen");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier3)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore1_moorhen".to_string())
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore1").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
 
         // The key doesn't exist in any of the stores yet.
         assert!(mgr.get::<TestItem>(&TestKeySpecifier4).unwrap().is_none());
@@ -1049,22 +1079,24 @@ mod tests {
             assert!(old_key.is_none());
 
             // Ensure the key now exists in `store`.
+            let key = mgr.get::<TestItem>(&TestKeySpecifier4).unwrap().unwrap();
+            assert_eq!(key.meta.item_id, "cormorant");
             assert_eq!(
-                mgr.get::<TestItem>(&TestKeySpecifier4)
-                    .unwrap()
-                    .map(|k| k.meta),
-                Some(format!("{store}_cormorant"))
+                key.meta.retrieved_from,
+                Some(KeystoreId::from_str(store).unwrap())
             );
+            assert_eq!(key.meta.is_generated, false);
         }
 
         // The key exists in all key stores, but if no keystore_id is specified, we return the
         // value from the first key store it is found in (in this case, Keystore1)
+        let key = mgr.get::<TestItem>(&TestKeySpecifier4).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "cormorant");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier4)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore1_cormorant".to_string())
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore1").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
     }
 
     #[test]
@@ -1089,12 +1121,13 @@ mod tests {
             true,
         )
         .unwrap();
+        let key = mgr.get::<TestItem>(&TestKeySpecifier1).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "coot");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier1)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore2_coot".to_string())
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
 
         // Try to remove the key from a non-existent key store
         assert!(mgr
@@ -1112,7 +1145,7 @@ mod tests {
         assert!(mgr
             .remove::<TestItem>(&TestKeySpecifier1, KeystoreSelector::Primary)
             .unwrap()
-            .is_none(),);
+            .is_none());
 
         // The key still exists in Keystore2
         assert!(mgr.secondary_stores[0]
@@ -1120,15 +1153,19 @@ mod tests {
             .unwrap());
 
         // Removing from Keystore2 should succeed.
-        assert_eq!(
-            mgr.remove::<TestItem>(
+        let removed_key = mgr
+            .remove::<TestItem>(
                 &TestKeySpecifier1,
-                KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap())
+                KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
             )
             .unwrap()
-            .map(|k| k.meta),
-            Some("keystore2_coot".to_string())
+            .unwrap();
+        assert_eq!(removed_key.meta.item_id, "coot");
+        assert_eq!(
+            removed_key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
         );
+        assert_eq!(removed_key.meta.is_generated, false);
 
         // The key doesn't exist in Keystore2 anymore
         assert!(!mgr.secondary_stores[0]
@@ -1138,6 +1175,7 @@ mod tests {
 
     #[test]
     fn keygen() {
+        let mut rng = FakeEntropicRng(testing_rng());
         let mgr = KeyMgrBuilder::default()
             .primary_store(Box::<Keystore1>::default())
             .build()
@@ -1155,14 +1193,14 @@ mod tests {
         assert!(mgr
             .get::<TestPublicKey>(&TestPublicKeySpecifier1)
             .unwrap()
-            .is_none(),);
+            .is_none());
 
         // Try to generate a new key (overwrite = false)
         let err = mgr
             .generate::<TestItem>(
                 &TestKeySpecifier1,
                 KeystoreSelector::Primary,
-                &mut testing_rng(),
+                &mut rng,
                 false,
             )
             .unwrap_err();
@@ -1170,47 +1208,55 @@ mod tests {
         assert!(matches!(err, crate::Error::KeyAlreadyExists));
 
         // The previous entry was not overwritten because overwrite = false
+        let key = mgr.get::<TestItem>(&TestKeySpecifier1).unwrap().unwrap();
+        assert_eq!(key.meta.item_id, "coot");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier1)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore1_coot".to_string())
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore1").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
 
         // We don't store public keys in the keystore
         assert!(mgr
             .get::<TestPublicKey>(&TestPublicKeySpecifier1)
             .unwrap()
-            .is_none(),);
+            .is_none());
 
         // Try to generate a new key (overwrite = true)
-        let key = mgr
+        let generated_key = mgr
             .generate::<TestItem>(
                 &TestKeySpecifier1,
                 KeystoreSelector::Primary,
-                &mut testing_rng(),
+                &mut rng,
                 true,
             )
             .unwrap();
 
-        assert_eq!(key.meta, "generated_test_key".to_string());
+        assert_eq!(generated_key.meta.item_id, "generated_test_key");
+        // Not set in a freshly generated key, because KeyMgr::generate()
+        // returns it straight away, without going through Keystore::get()
+        assert_eq!(generated_key.meta.retrieved_from, None);
+        assert_eq!(generated_key.meta.is_generated, true);
 
+        // Retrieve the inserted key
+        let retrieved_key = mgr.get::<TestItem>(&TestKeySpecifier1).unwrap().unwrap();
+        assert_eq!(retrieved_key.meta.item_id, "generated_test_key");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier1)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore1_generated_test_key".to_string())
+            retrieved_key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore1").unwrap())
         );
+        assert_eq!(retrieved_key.meta.is_generated, true);
 
         // We don't store public keys in the keystore
         assert!(mgr
             .get::<TestPublicKey>(&TestPublicKeySpecifier1)
             .unwrap()
-            .is_none(),);
+            .is_none());
     }
 
     #[test]
     fn get_or_generate() {
+        let mut rng = FakeEntropicRng(testing_rng());
         let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
         builder
@@ -1232,51 +1278,62 @@ mod tests {
         .unwrap();
 
         // The key already exists in keystore 2 so it won't be auto-generated.
+        let key = mgr
+            .get_or_generate::<TestItem>(&TestKeySpecifier1, KeystoreSelector::Primary, &mut rng)
+            .unwrap();
+        assert_eq!(key.meta.item_id, "coot");
         assert_eq!(
-            mgr.get_or_generate::<TestItem>(
-                &TestKeySpecifier1,
-                KeystoreSelector::Primary,
-                &mut testing_rng()
-            )
-            .unwrap()
-            .meta,
-            "keystore2_coot".to_string(),
+            key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore2").unwrap())
         );
+        assert_eq!(key.meta.is_generated, false);
 
         assert_eq!(
             mgr.get_entry::<TestItem>(&entry_desc1)
                 .unwrap()
                 .map(|k| k.meta),
-            Some("keystore2_coot".to_string())
+            Some(ItemMetadata {
+                item_id: "coot".to_string(),
+                retrieved_from: Some(keystore2.clone()),
+                is_generated: false,
+            })
         );
 
         // This key doesn't exist in any of the keystores, so it will be auto-generated and
         // inserted into keystore 3.
         let keystore3 = KeystoreId::from_str("keystore3").unwrap();
-        assert_eq!(
-            mgr.get_or_generate::<TestItem>(
+        let generated_key = mgr
+            .get_or_generate::<TestItem>(
                 &TestKeySpecifier2,
                 KeystoreSelector::Id(&keystore3),
-                &mut testing_rng()
+                &mut rng,
             )
-            .unwrap()
-            .meta,
-            "generated_test_key".to_string(),
-        );
+            .unwrap();
+        assert_eq!(generated_key.meta.item_id, "generated_test_key");
+        // Not set in a freshly generated key, because KeyMgr::get_or_generate()
+        // returns it straight away, without going through Keystore::get()
+        assert_eq!(generated_key.meta.retrieved_from, None);
+        assert_eq!(generated_key.meta.is_generated, true);
 
+        // Retrieve the inserted key
+        let retrieved_key = mgr.get::<TestItem>(&TestKeySpecifier2).unwrap().unwrap();
+        assert_eq!(retrieved_key.meta.item_id, "generated_test_key");
         assert_eq!(
-            mgr.get::<TestItem>(&TestKeySpecifier2)
-                .unwrap()
-                .map(|k| k.meta),
-            Some("keystore3_generated_test_key".to_string())
+            retrieved_key.meta.retrieved_from,
+            Some(KeystoreId::from_str("keystore3").unwrap())
         );
+        assert_eq!(retrieved_key.meta.is_generated, true);
 
         let entry_desc2 = entry_descriptor(TestKeySpecifier2, &keystore3);
         assert_eq!(
             mgr.get_entry::<TestItem>(&entry_desc2)
                 .unwrap()
                 .map(|k| k.meta),
-            Some("keystore3_generated_test_key".to_string()),
+            Some(ItemMetadata {
+                item_id: "generated_test_key".to_string(),
+                retrieved_from: Some(keystore3.clone()),
+                is_generated: true,
+            })
         );
 
         let arti_pat = KeyPathPattern::Arti("*".to_string());
@@ -1308,6 +1365,7 @@ mod tests {
         ) => {{
             use GenerateItem::*;
 
+            let mut rng = FakeEntropicRng(testing_rng());
             let mut builder = KeyMgrBuilder::default().primary_store(Box::<Keystore1>::default());
 
             builder
@@ -1327,7 +1385,7 @@ mod tests {
                     .generate::<TestItem>(
                         &TestKeySpecifier1,
                         KeystoreSelector::Primary,
-                        &mut testing_rng(),
+                        &mut rng,
                         false,
                     )
                     .unwrap();
@@ -1338,21 +1396,37 @@ mod tests {
                     .generate::<TestItem>(
                         &TestKeySpecifier2,
                         KeystoreSelector::Id(&KeystoreId::from_str("keystore2").unwrap()),
-                        &mut testing_rng(),
+                        &mut rng,
                         false,
                     )
                     .unwrap();
             }
 
             let make_certificate = move |subject_key: &TestItem, signed_with: &TestItem| {
-                let meta = format!(
-                    "a test cert for {} signed with {}",
-                    subject_key.meta, signed_with.meta
-                );
+                let meta = ItemMetadata {
+                    item_id: format!(
+                        "a test cert for {} signed with {}",
+                        subject_key.meta.item_id, signed_with.meta.item_id
+                    ),
+                    retrieved_from: None,
+                    is_generated: false,
+                };
 
-
-                let dummy_tor_cert = tor_key_forge::EncodedEd25519Cert::from_bytes(&[]);
-                let test_cert = CertData::TorEd25519Cert(dummy_tor_cert);
+                // Note: this is not really a cert for `subject_key` signed with the `signed_with`
+                // key!. The two are `TestItem`s and not keys, so we can't really generate a real
+                // cert from them. We can, however, pretend we did, for testing purposes.
+                // Eventually we might want to rewrite these tests to use real items
+                // (like the `ArtiNativeKeystore` tests)
+                let mut rng = FakeEntropicRng(testing_rng());
+                let keypair = ed25519::Keypair::generate(&mut rng);
+                let encoded_cert = Ed25519Cert::constructor()
+                    .cert_type(tor_cert::CertType::IDENTITY_V_SIGNING)
+                    .expiration(SystemTime::now() + Duration::from_secs(180))
+                    .signing_key(keypair.public_key().into())
+                    .cert_key(CertifiedKey::Ed25519(keypair.public_key().into()))
+                    .encode_and_sign(&keypair)
+                    .unwrap();
+                let test_cert = CertData::TorEd25519Cert(encoded_cert);
                 AlwaysValidCert(TestItem {
                     item: KeystoreItem::Cert(test_cert),
                     meta,
@@ -1364,7 +1438,7 @@ mod tests {
                     &spec,
                     &make_certificate,
                     KeystoreSelector::Primary,
-                    &mut testing_rng(),
+                    &mut rng,
                 );
 
             #[allow(unused_assignments)]
@@ -1385,17 +1459,16 @@ mod tests {
             if !has_error {
                 let (key, cert) = res.unwrap();
 
-
-                let expected_subj_key_meta = if $generate_subject_key == Yes {
-                    "keystore1_generated_test_key"
+                let expected_subj_key_id = if $generate_subject_key == Yes {
+                    "generated_test_key"
                 } else {
                     "generated_test_key"
                 };
 
-                assert_eq!(key.meta, expected_subj_key_meta);
+                assert_eq!(key.meta.item_id, expected_subj_key_id);
                 assert_eq!(
-                    cert.0.meta,
-                    format!("a test cert for {expected_subj_key_meta} signed with keystore2_generated_test_key")
+                    cert.0.meta.item_id,
+                    format!("a test cert for {} signed with generated_test_key", expected_subj_key_id)
                 );
             }
         }}
