@@ -1,28 +1,28 @@
 //! Module providing [`CtrlMsg`].
 
+use super::circuit::extender::CircuitExtender;
 use super::{
     CircuitHandshake, CloseStreamBehavior, MetaCellHandler, Reactor, ReactorResultChannel,
     RunOnceCmdInner, SendRelayCell,
 };
 use crate::crypto::binding::CircuitBinding;
 use crate::crypto::cell::{HopNum, InboundClientLayer, OutboundClientLayer, Tor1RelayCrypto};
-#[cfg(feature = "ntor_v3")]
 use crate::crypto::handshake::ntor_v3::{NtorV3Client, NtorV3PublicKey};
 use crate::stream::AnyCmdChecker;
 use crate::tunnel::circuit::celltypes::CreateResponse;
 use crate::tunnel::circuit::{path, CircParameters};
-use crate::tunnel::reactor::extender::CircuitExtender;
 use crate::tunnel::reactor::{NtorClient, ReactorError};
-use crate::tunnel::streammap;
+use crate::tunnel::{streammap, HopLocation, LegId, TargetHop};
 use crate::util::skew::ClockSkew;
 use crate::Result;
 use tor_cell::chancell::msg::HandshakeType;
+use tor_cell::relaycell::extend::NtorV3Extension;
 use tor_cell::relaycell::msg::{AnyRelayMsg, Sendme};
 use tor_cell::relaycell::{
     AnyRelayMsgOuter, RelayCellFormat, RelayCellFormatTrait, RelayCellFormatV0, StreamId,
     UnparsedRelayMsg,
 };
-use tor_error::Bug;
+use tor_error::{bad_api_usage, into_bad_api_usage, Bug};
 use tracing::trace;
 #[cfg(feature = "hs-service")]
 use {
@@ -31,7 +31,10 @@ use {
 };
 
 #[cfg(test)]
-use {crate::congestion::sendme::CircTag, crate::Error, tor_error::internal};
+use crate::congestion::sendme::CircTag;
+
+#[cfg(feature = "conflux")]
+use super::Circuit;
 
 use oneshot_fused_workaround as oneshot;
 
@@ -40,14 +43,15 @@ use crate::tunnel::circuit::{StreamMpscReceiver, StreamMpscSender};
 use tor_linkspec::{EncodedLinkSpec, OwnedChanTarget};
 
 use std::result::Result as StdResult;
+use std::sync::Arc;
 
 /// A message telling the reactor to do something.
 ///
-/// The difference between this and [`CtrlCmd`] is that `CtrlMsg`s
-/// are only handled when the reactor's `chan_sender` is ready to receive cells,
-/// whereas `CtrlCmd` are handled immediately as they arrive.
-///
 /// For each `CtrlMsg`, the reactor will send a cell on the underlying channel.
+///
+/// The difference between this and [`CtrlCmd`] is that `CtrlMsg`s
+/// cause the reactor to send cells on the reactor's `chan_sender`,
+/// whereas `CtrlCmd` do not.
 #[derive(educe::Educe)]
 #[educe(Debug)]
 pub(crate) enum CtrlMsg {
@@ -78,7 +82,6 @@ pub(crate) enum CtrlMsg {
         done: ReactorResultChannel<()>,
     },
     /// Extend a circuit by one hop, using the ntorv3 handshake.
-    #[cfg(feature = "ntor_v3")]
     ExtendNtorV3 {
         /// The peer that we're extending to.
         ///
@@ -98,7 +101,7 @@ pub(crate) enum CtrlMsg {
     /// Allocates a stream ID, and sends the provided message to that hop.
     BeginStream {
         /// The hop number to begin the stream with.
-        hop_num: HopNum,
+        hop: TargetHop,
         /// The message to send.
         message: AnyRelayMsg,
         /// A channel to send messages on this stream down.
@@ -111,7 +114,7 @@ pub(crate) enum CtrlMsg {
         /// A channel to receive messages to send on this stream from.
         rx: StreamMpscReceiver<AnyRelayMsg>,
         /// Oneshot channel to notify on completion, with the allocated stream ID.
-        done: ReactorResultChannel<StreamId>,
+        done: ReactorResultChannel<(StreamId, HopLocation, RelayCellFormat)>,
         /// A `CmdChecker` to keep track of which message types are acceptable.
         cmd_checker: AnyCmdChecker,
     },
@@ -125,7 +128,7 @@ pub(crate) enum CtrlMsg {
     #[cfg(feature = "hs-service")]
     ClosePendingStream {
         /// The hop number the stream is on.
-        hop_num: HopNum,
+        hop: HopLocation,
         /// The stream ID to send the END for.
         stream_id: StreamId,
         /// The END message to send, if any.
@@ -164,7 +167,9 @@ pub(crate) enum CtrlMsg {
         /// The stream ID to send a SENDME for.
         stream_id: StreamId,
         /// The hop number the stream is on.
-        hop_num: HopNum,
+        hop: HopLocation,
+        /// A sender that we use to tell the caller that the SENDME was sent.
+        sender: oneshot::Sender<Result<()>>,
     },
     /// Get the clock skew claimed by the first hop of the circuit.
     FirstHopClockSkew {
@@ -176,8 +181,7 @@ pub(crate) enum CtrlMsg {
 /// A message telling the reactor to do something.
 ///
 /// The difference between this and [`CtrlMsg`] is that `CtrlCmd`s
-/// are handled even if the reactor's `chan_sender` is not ready to receive cells.
-/// Another difference is that `CtrlCmd`s never cause cells to sent on the channel,
+/// never cause cells to sent on the channel,
 /// while `CtrlMsg`s potentially do: `CtrlMsg`s are mapped to [`RunOnceCmdInner`] commands,
 /// some of which instruct the reactor to send cells down the channel.
 #[derive(educe::Educe)]
@@ -222,6 +226,11 @@ pub(crate) enum CtrlCmd {
         #[cfg(feature = "hs-service")]
         filter: Box<dyn IncomingStreamRequestFilter>,
     },
+    /// Get the leg ID and path for each leg of the tunnel.
+    QueryLegs {
+        /// Oneshot channel to notify on completion.
+        done: ReactorResultChannel<Vec<(LegId, Arc<path::Path>)>>,
+    },
     /// (tests only) Add a hop to the list of hops on this circuit, with dummy cryptography.
     #[cfg(test)]
     AddFakeHop {
@@ -237,6 +246,17 @@ pub(crate) enum CtrlCmd {
         hop: HopNum,
         done: ReactorResultChannel<(u32, Vec<CircTag>)>,
     },
+    /// Shut down the reactor, and return the underlying [`Circuit`],
+    /// if the tunnel is not multi-path.
+    ///
+    /// Returns an error if called on a multi-path reactor.
+    #[cfg(feature = "conflux")]
+    #[allow(unused)] // TODO(conflux)
+    ShutdownAndReturnCircuit {
+        /// Oneshot channel to return the underlying [`Circuit`],
+        /// or an error if the reactor's tunnel is multi-path.
+        answer: oneshot::Sender<StdResult<Circuit, Bug>>,
+    },
 }
 
 /// A control message handler object. Keep a reference to the Reactor tying its lifetime to it.
@@ -263,7 +283,7 @@ impl<'a> ControlHandler<'a> {
                 if self.reactor.circuits.len() == 1 {
                     // This should've been handled in Reactor::run_once()
                     // (ControlHandler::handle_msg() is never called before wait_for_create()).
-                    debug_assert!(!self.reactor.circuits.single_leg_mut()?.hops.is_empty());
+                    debug_assert!(self.reactor.circuits.single_leg()?.1.has_hops());
                     // Don't care if the receiver goes away
                     let _ = done.send(Err(tor_error::bad_api_usage!(
                         "cannot create first hop twice"
@@ -286,7 +306,7 @@ impl<'a> ControlHandler<'a> {
                 params,
                 done,
             } => {
-                let Ok(circ) = self.reactor.circuits.single_leg_mut() else {
+                let Ok((_id, circ)) = self.reactor.circuits.single_leg_mut() else {
                     // Don't care if the receiver goes away
                     let _ = done.send(Err(tor_error::bad_api_usage!(
                         "cannot extend multipath tunnel"
@@ -318,7 +338,6 @@ impl<'a> ControlHandler<'a> {
 
                 Ok(Some(RunOnceCmdInner::Send { cell, done: None }))
             }
-            #[cfg(feature = "ntor_v3")]
             CtrlMsg::ExtendNtorV3 {
                 peer_id,
                 public_key,
@@ -326,7 +345,7 @@ impl<'a> ControlHandler<'a> {
                 params,
                 done,
             } => {
-                let Ok(circ) = self.reactor.circuits.single_leg_mut() else {
+                let Ok((_id, circ)) = self.reactor.circuits.single_leg_mut() else {
                     // Don't care if the receiver goes away
                     let _ = done.send(Err(tor_error::bad_api_usage!(
                         "cannot extend multipath tunnel"
@@ -336,12 +355,40 @@ impl<'a> ControlHandler<'a> {
                     return Ok(None);
                 };
 
-                // TODO #1067: support negotiating other formats.
+                // TODO #1067, TODO #1947: support negotiating other formats.
                 /// Local type alias to ensure consistency below.
                 type Rcf = RelayCellFormatV0;
 
-                // TODO: Set extensions, e.g. based on `params`.
-                let client_extensions = [];
+                // Set the client extensions.
+                // allow 'unused_mut' because of the combinations of `cfg` conditions below
+                #[allow(unused_mut)]
+                let mut client_extensions = Vec::new();
+
+                if params.ccontrol.is_enabled() {
+                    cfg_if::cfg_if! {
+                        if #[cfg(feature = "flowctl-cc")] {
+                            // TODO(arti#88): We have an `if false` in `exit_circparams_from_netparams`
+                            // which should prevent the above `is_enabled()` from ever being true,
+                            // even with the "flowctl-cc" feature enabled:
+                            // https://gitlab.torproject.org/tpo/core/arti/-/merge_requests/2932#note_3191196
+                            // The panic here is so that CI tests will hopefully catch if congestion
+                            // control is unexpectedly enabled.
+                            // We should remove this panic once xon/xoff flow is supported.
+                            #[cfg(not(test))]
+                            panic!("Congestion control is enabled on this circuit, but we don't yet support congestion control");
+
+                            #[allow(unreachable_code)]
+                            client_extensions.push(NtorV3Extension::RequestCongestionControl);
+                        } else {
+                            return Err(
+                                tor_error::internal!(
+                                    "Congestion control is enabled on this circuit, but 'flowctl-cc' feature is not enabled"
+                                )
+                                .into()
+                            );
+                        }
+                    }
+                }
 
                 let (extender, cell) =
                     CircuitExtender::<NtorV3Client, Tor1RelayCrypto<Rcf>, _, _>::begin(
@@ -361,54 +408,117 @@ impl<'a> ControlHandler<'a> {
 
                 Ok(Some(RunOnceCmdInner::Send { cell, done: None }))
             }
-            // TODO(conflux): this should specify which leg this stream is on
-            // (currently we assume it's the primary leg)
             CtrlMsg::BeginStream {
-                hop_num,
+                hop,
                 message,
                 sender,
                 rx,
                 done,
                 cmd_checker,
             } => {
-                // TODO(conflux)/TODO(#1857): should this take a leg_id argument?
-                // Currently, we always begin streams on the primary leg
-                let circ = self.reactor.circuits.primary_leg_mut()?;
+                // If resolving the hop fails,
+                // we want to report an error back to the initiator and not shut down the reactor.
+                let hop_location = match self.reactor.resolve_target_hop(hop) {
+                    Ok(x) => x,
+                    Err(e) => {
+                        let e = into_bad_api_usage!("Could not resolve {hop:?}")(e);
+                        // don't care if receiver goes away
+                        let _ = done.send(Err(e.into()));
+                        return Ok(None);
+                    }
+                };
+                let (leg_id, hop_num) = match self.reactor.resolve_hop_location(hop_location) {
+                    Ok(x) => x,
+                    Err(e) => {
+                        let e = into_bad_api_usage!("Could not resolve {hop_location:?}")(e);
+                        // don't care if receiver goes away
+                        let _ = done.send(Err(e.into()));
+                        return Ok(None);
+                    }
+                };
+                let circ = match self.reactor.circuits.leg_mut(leg_id) {
+                    Some(x) => x,
+                    None => {
+                        let e = bad_api_usage!("Circuit leg {leg_id:?} does not exist");
+                        // don't care if receiver goes away
+                        let _ = done.send(Err(e.into()));
+                        return Ok(None);
+                    }
+                };
+
                 let cell = circ.begin_stream(hop_num, message, sender, rx, cmd_checker)?;
-                Ok(Some(RunOnceCmdInner::BeginStream { cell, done }))
+                Ok(Some(RunOnceCmdInner::BeginStream {
+                    cell,
+                    hop: hop_location,
+                    done,
+                }))
             }
-            // TODO(conflux): this should specify which leg this stream is on
-            // (currently we assume it's the primary leg)
             #[cfg(feature = "hs-service")]
             CtrlMsg::ClosePendingStream {
-                hop_num,
+                hop,
                 stream_id,
                 message,
                 done,
-            } => {
-                // TODO(conflux): what to do if there are multiple legs?
-                // The hop_num won't be enough to identify the hop the stream is with
-                Ok(Some(RunOnceCmdInner::CloseStream {
-                    hop_num,
-                    sid: stream_id,
-                    behav: message,
-                    reason: streammap::TerminateReason::ExplicitEnd,
-                    done: Some(done),
-                }))
-            }
-            // TODO(conflux): this should specify which leg to send the msg on
-            // (currently we send it down the primary leg)
-            //
+            } => Ok(Some(RunOnceCmdInner::CloseStream {
+                hop,
+                sid: stream_id,
+                behav: message,
+                reason: streammap::TerminateReason::ExplicitEnd,
+                done: Some(done),
+            })),
             // TODO(#1860): remove stream-level sendme support
-            CtrlMsg::SendSendme { stream_id, hop_num } => {
+            CtrlMsg::SendSendme {
+                stream_id,
+                hop,
+                sender,
+            } => {
+                // If resolving the hop fails,
+                // we want to report an error back to the initiator and not shut down the reactor.
+                let (leg_id, hop_num) = match self.reactor.resolve_hop_location(hop) {
+                    Ok(x) => x,
+                    Err(e) => {
+                        let e = into_bad_api_usage!("Could not resolve hop {hop:?}")(e);
+                        // don't care if receiver goes away
+                        let _ = sender.send(Err(e.into()));
+                        return Ok(None);
+                    }
+                };
+
+                // Congestion control decides if we can send stream level SENDMEs or not.
+                let sendme_required = match self.reactor.uses_stream_sendme(leg_id, hop_num) {
+                    Some(x) => x,
+                    None => {
+                        // don't care if receiver goes away
+                        let _ = sender.send(Err(bad_api_usage!(
+                            "Unknown hop {hop_num:?} on leg {leg_id:?}"
+                        )
+                        .into()));
+                        return Ok(None);
+                    }
+                };
+
+                if !sendme_required {
+                    // don't care if receiver goes away
+                    let _ = sender.send(Ok(()));
+                    return Ok(None);
+                }
+
                 let sendme = Sendme::new_empty();
                 let cell = AnyRelayMsgOuter::new(Some(stream_id), sendme.into());
+
                 let cell = SendRelayCell {
                     hop: hop_num,
                     early: false,
                     cell,
                 };
-                Ok(Some(RunOnceCmdInner::Send { cell, done: None }))
+                // TODO(conflux): We need to pass the leg id in `RunOnceCmdInner::Send`, but we
+                // can't add this to `RunOnceCmdInner` until the stream map knows about legs.
+                // `Reactor::ready_streams_iterator` now knows the leg id, so we should be able to
+                // use that.
+                Ok(Some(RunOnceCmdInner::Send {
+                    cell,
+                    done: Some(sender),
+                }))
             }
             // TODO(conflux): this should specify which leg to send the msg on
             // (currently we send it down the primary leg)
@@ -451,7 +561,7 @@ impl<'a> ControlHandler<'a> {
     pub(super) fn handle_cmd(&mut self, msg: CtrlCmd) -> StdResult<(), ReactorError> {
         trace!("{}: reactor received {:?}", self.reactor.unique_id, msg);
         match msg {
-            CtrlCmd::Shutdown => Err(ReactorError::Shutdown),
+            CtrlCmd::Shutdown => self.reactor.handle_shutdown().map(|_| ()),
             #[cfg(feature = "hs-common")]
             #[allow(unreachable_code)]
             CtrlCmd::ExtendVirtual {
@@ -466,12 +576,17 @@ impl<'a> ControlHandler<'a> {
                 // describe why the virtual hop was added, or something?
                 let peer_id = path::HopDetail::Virtual;
 
-                // TODO(conflux): the error should probably be sent via the done channel?
-                // (it should probably not crash the reactor)
-                self.reactor
-                    .circuits
-                    .single_leg_mut()?
-                    .add_hop(format, peer_id, outbound, inbound, binding, &params);
+                let Ok((_id, leg)) = self.reactor.circuits.single_leg_mut() else {
+                    // Don't care if the receiver goes away
+                    let _ = done.send(Err(tor_error::bad_api_usage!(
+                        "cannot extend multipath tunnel"
+                    )
+                    .into()));
+
+                    return Ok(());
+                };
+
+                leg.add_hop(format, peer_id, outbound, inbound, binding, &params)?;
                 let _ = done.send(Ok(()));
 
                 Ok(())
@@ -501,6 +616,17 @@ impl<'a> ControlHandler<'a> {
 
                 Ok(())
             }
+            CtrlCmd::QueryLegs { done } => {
+                let ret = self
+                    .reactor
+                    .circuits
+                    .legs()
+                    .map(|(id, leg)| (id, leg.path()))
+                    .collect();
+                // TODO(conflux): Consider adding the leg id to the `Path`.
+                let _ = done.send(Ok(ret));
+                Ok(())
+            }
             #[cfg(test)]
             CtrlCmd::AddFakeHop {
                 relay_cell_format,
@@ -509,31 +635,46 @@ impl<'a> ControlHandler<'a> {
                 params,
                 done,
             } => {
-                self.reactor.circuits.single_leg_mut()?.handle_add_fake_hop(
-                    relay_cell_format,
-                    fwd_lasthop,
-                    rev_lasthop,
-                    &params,
-                    done,
-                );
+                let Ok((_id, leg)) = self.reactor.circuits.single_leg_mut() else {
+                    // Don't care if the receiver goes away
+                    let _ = done.send(Err(tor_error::bad_api_usage!(
+                        "cannot add fake hop to multipath tunnel"
+                    )
+                    .into()));
+
+                    return Ok(());
+                };
+
+                leg.handle_add_fake_hop(relay_cell_format, fwd_lasthop, rev_lasthop, &params, done);
 
                 Ok(())
             }
             #[cfg(test)]
             CtrlCmd::QuerySendWindow { hop, done } => {
-                let _ = done.send(
-                    if let Some(hop) = self.reactor.circuits.single_leg_mut()?.hop_mut(hop) {
-                        Ok(hop.ccontrol.send_window_and_expected_tags())
-                    } else {
-                        Err(Error::from(internal!(
-                            "received QuerySendWindow for unknown hop {}",
-                            hop.display()
-                        )))
-                    },
-                );
+                // Immediately invoked function means that errors will be sent to the channel.
+                let _ = done.send((|| {
+                    let (_id, leg) =
+                        self.reactor
+                            .circuits
+                            .single_leg_mut()
+                            .map_err(into_bad_api_usage!(
+                                "cannot query send window of multipath tunnel"
+                            ))?;
+
+                    let hop = leg.hop_mut(hop).ok_or(bad_api_usage!(
+                        "received QuerySendWindow for unknown hop {}",
+                        hop.display()
+                    ))?;
+
+                    Ok(hop.send_window_and_expected_tags())
+                })());
 
                 Ok(())
             }
+            #[cfg(feature = "conflux")]
+            CtrlCmd::ShutdownAndReturnCircuit { answer } => {
+                self.reactor.handle_shutdown_and_return_circuit(answer)
+            }
         }
     }
 }
