@@ -3,6 +3,7 @@
 pub(crate) mod ct;
 pub(crate) mod err;
 pub(crate) mod keyed_futures_unordered;
+pub(crate) mod oneshot_broadcast;
 pub(crate) mod skew;
 pub(crate) mod sometimes_unbounded_sink;
 pub(crate) mod stream_poll_set;
@@ -31,3 +32,12 @@ pub(crate) trait SinkExt<T>: Sink<T> {
     }
 }
 impl<T, S: Sink<T>> SinkExt<T> for S {}
+
+/// Convenience alias for
+/// [`memquota::SpecificAccount::new_noop()`](crate::memquota::SpecificAccount::new_noop())
+///
+/// Available only in tests, which makes diff hunks which call this more obviously correct.
+#[cfg(any(test, feature = "testing"))]
+pub(crate) fn fake_mq<A: crate::memquota::SpecificAccount>() -> A {
+    A::new_noop()
+}
