@@ -67,8 +67,9 @@ pub mod unix;
 #[cfg(any(feature = "async-std", feature = "tokio"))]
 use std::io;
 pub use traits::{
-    BlockOn, CertifiedConn, CoarseTimeProvider, NetStreamListener, NetStreamProvider, Runtime,
-    SleepProvider, StreamOps, TlsProvider, UdpProvider, UdpSocket, UnsupportedStreamOp,
+    Blocking, CertifiedConn, CoarseTimeProvider, NetStreamListener, NetStreamProvider,
+    NoOpStreamOpsHandle, Runtime, SleepProvider, StreamOps, TlsProvider, ToplevelBlockOn,
+    ToplevelRuntime, UdpProvider, UdpSocket, UnsupportedStreamOp,
 };
 
 pub use coarse_time::{CoarseDuration, CoarseInstant, RealCoarseTimeProvider};
@@ -110,6 +111,10 @@ use tokio as preferred_backend_mod;
 /// performance.
 /// If `native_tls` and `rustls` are both available, we prefer `native_tls` since
 /// it has been used in Arti for longer.
+///
+/// The process [**may not fork**](crate#do-not-fork)
+/// (except, very carefully, before exec)
+/// after creating this or any other `Runtime`.
 #[cfg(all(
     any(feature = "native-tls", feature = "rustls"),
     any(feature = "async-std", feature = "tokio")
@@ -157,6 +162,9 @@ impl PreferredRuntime {
     /// If the `tor-rtcompat` crate was compiled with `tokio` support,
     /// this function will never return a runtime based on `async_std`.
     ///
+    /// The process [**may not fork**](crate#do-not-fork)
+    /// (except, very carefully, before exec)
+    /// after creating this or any other `Runtime`.
     //
     // ## Note to Arti developers
     //
@@ -182,6 +190,10 @@ impl PreferredRuntime {
     ///
     /// If you need more fine-grained control over a runtime, you can create it
     /// using an appropriate builder type or function.
+    ///
+    /// The process [**may not fork**](crate#do-not-fork)
+    /// (except, very carefully, before exec)
+    /// after creating this or any other `Runtime`.
     //
     // ## Note to Arti developers
     //
@@ -363,8 +375,8 @@ macro_rules! test_with_one_runtime {
 ))]
 mod test {
     #![allow(clippy::unwrap_used, clippy::unnecessary_wraps)]
-    use crate::Runtime;
     use crate::SleepProviderExt;
+    use crate::ToplevelRuntime;
 
     use crate::traits::*;
 
@@ -378,7 +390,7 @@ mod test {
 
     // Test "sleep" with a tiny delay, and make sure that at least that
     // much delay happens.
-    fn small_delay<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_delay<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let i1 = Instant::now();
@@ -391,7 +403,7 @@ mod test {
     }
 
     // Try a timeout operation that will succeed.
-    fn small_timeout_ok<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_timeout_ok<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let one_day = Duration::from_secs(86400);
@@ -402,7 +414,7 @@ mod test {
     }
 
     // Try a timeout operation that will time out.
-    fn small_timeout_expire<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn small_timeout_expire<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         use futures::future::pending;
 
         let rt = runtime.clone();
@@ -421,7 +433,7 @@ mod test {
     //
     // NOTE: This test will fail if the clock jumps a lot while it's
     // running.  We should use simulated time instead.
-    fn tiny_wallclock<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn tiny_wallclock<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let rt = runtime.clone();
         runtime.block_on(async {
             let i1 = Instant::now();
@@ -442,7 +454,7 @@ mod test {
     // Try connecting to ourself and sending a little data.
     //
     // NOTE: requires Ipv4 localhost.
-    fn self_connect_tcp<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn self_connect_tcp<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -475,7 +487,7 @@ mod test {
     // Try connecting to ourself and sending a little data.
     //
     // NOTE: requires Ipv4 localhost.
-    fn self_connect_udp<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn self_connect_udp<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -510,7 +522,7 @@ mod test {
     //
     // We launch a few connections and make sure that we can read data on
     // them.
-    fn listener_stream<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn listener_stream<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         let localhost = SocketAddrV4::new(Ipv4Addr::LOCALHOST, 0);
         let rt1 = runtime.clone();
 
@@ -559,7 +571,7 @@ mod test {
     //
     // Note that since we don't have async tls server support yet, I'm just
     // going to use a thread.
-    fn simple_tls<R: Runtime>(runtime: &R) -> IoResult<()> {
+    fn simple_tls<R: ToplevelRuntime>(runtime: &R) -> IoResult<()> {
         /*
          A simple expired self-signed rsa-2048 certificate.
 
