@@ -196,11 +196,11 @@ impl<'b, O, P: ReadPrecision> Finished<'b, O, P> {
 
     /// Return the output, and the following already-read data as a `Vec`
     ///
-    /// The `Vec` is quite likely to have a considerably larger capacity than contnets.
+    /// The `Vec` is quite likely to have a considerably larger capacity than contents.
     /// (Its capacity is usually the original buffer size, when the `Buffer` was created.)
     ///
     /// The `Buffer` should not be discarded after calling this;
-    /// it will not be useable.
+    /// it will not be usable.
     //
     // Ideally, this would *consume* the Buffer.  But that would mean that
     // `step` would have to take and return the buffer,
@@ -337,7 +337,7 @@ impl<P: ReadPrecision> Buffer<P> {
     ///
     ///  * `buf[..filled]` should contain data already read from the peer
     ///  * `buf[filled..]` should be zero (or other innocuous data),
-    ///                    and will not be used (except if there are bugs)
+    ///    and will not be used (except if there are bugs)
     ///
     /// Using this and `into_parts` to obtain a `Buffer`
     /// with a differetn the read precision (different `P` type parameter)
@@ -639,7 +639,7 @@ pub trait Handshake: HandshakeImpl + HasHandshakeOutput<Self::Output> {
                 reply,
                 finished: false,
             })),
-            Ok(ImplNextStep::Finished {}) => Ok(Ok(Action {
+            Ok(ImplNextStep::Finished) => Ok(Ok(Action {
                 drain,
                 reply: vec![],
                 finished: true,
